from transformers import AutoImageProcessor, AutoModelForImageClassification,DefaultDataCollator, Trainer, TrainingArguments, pipeline
from datasets import load_dataset
from torchvision.transforms import RandomResizedCrop, Compose, Normalize, ToTensor
import evaluate
import numpy as np

def load_image_dataset(data_dir, test_size=0.2):
    dataset = load_dataset("imagefolder", data_dir=data_dir, split="train")
    dataset = dataset.train_test_split(test_size=test_size)
    labels = dataset["train"].features["label"].names
    label2id, id2label = dict(), dict()
    for i, label in enumerate(labels):
        label2id[label] = str(i)
        id2label[str(i)] = label
    return dataset, label2id, id2label

def create_transforms(image_processor):
    normalize = Normalize(mean=image_processor.image_mean, std=image_processor.image_std)
    size = (
        image_processor.size["shortest_edge"]
        if "shortest_edge" in image_processor.size
        else (image_processor.size["height"], image_processor.size["width"])
    )
    _transforms = Compose([RandomResizedCrop(size), ToTensor(), normalize])

    def transforms(examples):
        examples["pixel_values"] = [_transforms(img.convert("RGB")) for img in examples["image"]]
        del examples["image"]
        return examples

    return transforms

def train_image_classifier(data_dir, test_size=0.2, checkpoint="google/vit-base-patch16-224-in21k", num_labels=None):
    dataset, label2id, id2label = load_image_dataset(data_dir, test_size)
    image_processor = AutoImageProcessor.from_pretrained(checkpoint)
    transforms = create_transforms(image_processor)
    dataset = dataset.with_transform(transforms)

    data_collator = DefaultDataCollator()

    accuracy = evaluate.load("accuracy")
    def compute_metrics(eval_pred):
        predictions, labels = eval_pred
        predictions = np.argmax(predictions, axis=1)
        return accuracy.compute(predictions=predictions, references=labels)

    model = AutoModelForImageClassification.from_pretrained(
        checkpoint,
        num_labels=num_labels if num_labels else len(dataset["train"].features["label"].names),
        id2label=id2label,
        label2id=label2id,
    )

    training_args = TrainingArguments(
        output_dir="my_awesome_food_model",
        remove_unused_columns=False,
        evaluation_strategy="epoch",
        save_strategy="epoch",
        learning_rate=5e-5,
        per_device_train_batch_size=16,
        gradient_accumulation_steps=4,
        per_device_eval_batch_size=16,
        num_train_epochs=3,
        warmup_ratio=0.1,
        logging_steps=10,
        load_best_model_at_end=True,
        metric_for_best_model="accuracy",
    )

    trainer = Trainer(
        model=model,
        args=training_args,
        data_collator=data_collator,
        train_dataset=dataset["train"],
        eval_dataset=dataset["test"],
        tokenizer=image_processor,
        compute_metrics=compute_metrics,
    )

    trainer.train()

def classify_image(image_path, model="my_awesome_food_model"):
    classifier = pipeline("image-classification", model=model)
    return classifier(image_path)
